/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010 University of Bonn. All rights reserved.
 * Please see the LICENSE file or "Copyright notice" in builder.cpp for details.
 */

/*
 * ScaleBoxAction.cpp
 *
 *  Created on: May 8, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "atom.hpp"
#include "CodePatterns/Log.hpp"
#include "LinearAlgebra/RealSpaceMatrix.hpp"
#include "LinearAlgebra/Vector.hpp"
#include "World.hpp"
#include "Box.hpp"

#include <iostream>
#include <string>
#include <vector>

#include "Actions/WorldAction/ScaleBoxAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "ScaleBoxAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
Action::state_ptr WorldScaleBoxAction::performCall() {
  double x[NDIM];

  // obtain information
  getParametersfromValueStorage();

  // scale atoms
  for (int i=0;i<NDIM;i++)
    x[i] = params.Scaler[i];
  std::vector<atom*> AllAtoms = World::getInstance().getAllAtoms();
  for(std::vector<atom*>::iterator AtomRunner = AllAtoms.begin(); AtomRunner != AllAtoms.end(); ++AtomRunner) {
    (*AtomRunner)->ScaleAll(x);
  }

  // scale box
  RealSpaceMatrix M = World::getInstance().getDomain().getM();
  RealSpaceMatrix scale;
  for (int i=0;i<NDIM;i++) {
    scale.at(i,i) = params.Scaler[i];
  }
  M *= scale;
  World::getInstance().setDomain(M);

  // give final box size
  LOG(0, "Box domain is now " << World::getInstance().getDomain().getM());

  // create undo state
  WorldScaleBoxState *UndoState = new WorldScaleBoxState(params);

  return Action::state_ptr(UndoState);
}

Action::state_ptr WorldScaleBoxAction::performUndo(Action::state_ptr _state) {
  WorldScaleBoxState *state = assert_cast<WorldScaleBoxState*>(_state.get());

  // scale back atoms
  double x[NDIM];
  for (int i=0;i<NDIM;i++)
    x[i] = 1./state->params.Scaler[i];
  vector<atom*> AllAtoms = World::getInstance().getAllAtoms();
  for(vector<atom*>::iterator AtomRunner = AllAtoms.begin(); AtomRunner != AllAtoms.end(); ++AtomRunner) {
    (*AtomRunner)->ScaleAll(x);
  }

  // scale back box
  RealSpaceMatrix M = World::getInstance().getDomain().getM();
  RealSpaceMatrix scale;
  for (int i=0;i<NDIM;i++) {
    scale.at(i,i) = 1./state->params.Scaler[i];
  }
  M *= scale;
  World::getInstance().setDomain(M);

  // give final box size
  LOG(0, "Box domain restored to " << World::getInstance().getDomain().getM());

  return Action::state_ptr(_state);
}

Action::state_ptr WorldScaleBoxAction::performRedo(Action::state_ptr _state){
  WorldScaleBoxState *state = assert_cast<WorldScaleBoxState*>(_state.get());

  // scale atoms
  double x[NDIM];
  for (int i=0;i<NDIM;i++)
    x[i] = state->params.Scaler[i];
  vector<atom*> AllAtoms = World::getInstance().getAllAtoms();
  for(vector<atom*>::iterator AtomRunner = AllAtoms.begin(); AtomRunner != AllAtoms.end(); ++AtomRunner) {
    (*AtomRunner)->ScaleAll(x);
  }

  // scale box
  RealSpaceMatrix M = World::getInstance().getDomain().getM();
  RealSpaceMatrix scale;
  for (int i=0;i<NDIM;i++) {
    scale.at(i,i) = state->params.Scaler[i];
  }
  M *= scale;
  World::getInstance().setDomain(M);

  // give final box size
  LOG(0, "Box domain is again " << World::getInstance().getDomain().getM());

  return Action::state_ptr(_state);
}

bool WorldScaleBoxAction::canUndo() {
  return true;
}

bool WorldScaleBoxAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
